MODULE iceitd
   !!======================================================================
   !!                       ***  MODULE iceitd ***
   !!   sea-ice : ice thickness distribution
   !!======================================================================
   !! History :   -   !          (W. H. Lipscomb and E.C. Hunke) CICE (c) original code
   !!            3.0  ! 2005-12  (M. Vancoppenolle) adaptation to LIM-3
   !!             -   ! 2006-06  (M. Vancoppenolle) adaptation to include salt, age
   !!             -   ! 2007-04  (M. Vancoppenolle) Mass conservation checked
   !!----------------------------------------------------------------------
#if defined key_lim3
   !!----------------------------------------------------------------------
   !!   'key_lim3'                                       ESIM sea-ice model
   !!----------------------------------------------------------------------
   !!   ice_itd_init   : read ice thicknesses mean and min from namelist
   !!   ice_itd_rem    : redistribute ice thicknesses after thermo growth and melt
   !!   ice_itd_reb    : rebin ice thicknesses into bounded categories
   !!----------------------------------------------------------------------
   USE dom_oce        ! ocean domain
   USE phycst         ! physical constants 
   USE ice1D          ! sea-ice: thermodynamic variables
   USE ice            ! sea-ice: variables
   USE icectl         ! sea-ice: conservation tests
   USE icetab         ! sea-ice: convert 1D<=>2D
   !
   USE in_out_manager ! I/O manager
   USE lib_mpp        ! MPP library
   USE lib_fortran    ! fortran utilities (glob_sum + no signed zero)
   USE prtctl         ! Print control

   IMPLICIT NONE
   PRIVATE

   PUBLIC   ice_itd_init  ! called in icestp
   PUBLIC   ice_itd_rem   ! called in icethd
   PUBLIC   ice_itd_reb   ! called in icecor

   ! ** namelist (namitd) **
   REAL(wp) ::   rn_himean   ! mean thickness of the domain

   !!----------------------------------------------------------------------
   !! NEMO/ICE 4.0 , NEMO Consortium (2017)
   !! $Id: iceitd.F90 8420 2017-08-08 12:18:46Z clem $
   !! Software governed by the CeCILL licence     (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS

   SUBROUTINE ice_itd_rem( kt )
      !!------------------------------------------------------------------
      !!                ***  ROUTINE ice_itd_rem ***
      !!
      !! ** Purpose :   computes the redistribution of ice thickness
      !!                after thermodynamic growth of ice thickness
      !!
      !! ** Method  :   Linear remapping 
      !!
      !! References :   W.H. Lipscomb, JGR 2001
      !!------------------------------------------------------------------
      INTEGER , INTENT (in) ::   kt      ! Ocean time step 
      !
      INTEGER  ::   ji, jj, jl, jcat     ! dummy loop index
      INTEGER  ::   nidx2                ! local integer
      REAL(wp) ::   zx1, zwk1, zdh0, zetamin, zdamax   ! local scalars
      REAL(wp) ::   zx2, zwk2, zda0, zetamax           !   -      -
      REAL(wp) ::   zx3        
      REAL(wp) ::   zslope          ! used to compute local thermodynamic "speeds"

      INTEGER , DIMENSION(jpij)       ::   idxice2         ! compute remapping or not
      INTEGER , DIMENSION(jpij,jpl-1) ::   jdonor          ! donor category index
      REAL(wp), DIMENSION(jpij,jpl)   ::   zdhice          ! ice thickness increment
      REAL(wp), DIMENSION(jpij,jpl)   ::   g0, g1          ! coefficients for fitting the line of the ITD
      REAL(wp), DIMENSION(jpij,jpl)   ::   hL, hR          ! left and right boundary for the ITD for each thickness
      REAL(wp), DIMENSION(jpij,jpl-1) ::   zdaice, zdvice  ! local increment of ice area and volume
      REAL(wp), DIMENSION(jpij)       ::   zhb0, zhb1      ! category boundaries for thinnes categories
      REAL(wp), DIMENSION(jpij,0:jpl) ::   zhbnew          ! new boundaries of ice categories
      !!------------------------------------------------------------------

      IF( kt == nit000 .AND. lwp )   WRITE(numout,*) '-- ice_itd_rem: remapping ice thickness distribution' 

      IF( ln_icediachk )   CALL ice_cons_hsm(0, 'iceitd_rem', rdiag_v, rdiag_s, rdiag_t, rdiag_fv, rdiag_fs, rdiag_ft)

      !-----------------------------------------------------------------------------------------------
      !  1) Identify grid cells with ice
      !-----------------------------------------------------------------------------------------------
      nidx = 0   ;   idxice(:) = 0
      DO jj = 1, jpj
         DO ji = 1, jpi
            IF ( at_i(ji,jj) > epsi10 ) THEN
               nidx         = nidx + 1
               idxice( nidx ) = (jj - 1) * jpi + ji
            ENDIF
         END DO
      END DO
      
      !-----------------------------------------------------------------------------------------------
      !  2) Compute new category boundaries
      !-----------------------------------------------------------------------------------------------
      IF( nidx > 0 ) THEN

         zdhice(:,:) = 0._wp
         zhbnew(:,:) = 0._wp

         CALL tab_3d_2d( nidx, idxice(1:nidx), h_i_2d (1:nidx,1:jpl), h_i   )
         CALL tab_3d_2d( nidx, idxice(1:nidx), h_ib_2d(1:nidx,1:jpl), h_i_b )
         CALL tab_3d_2d( nidx, idxice(1:nidx), a_i_2d  (1:nidx,1:jpl), a_i    )
         CALL tab_3d_2d( nidx, idxice(1:nidx), a_ib_2d (1:nidx,1:jpl), a_i_b  )

         DO jl = 1, jpl
            ! Compute thickness change in each ice category
            DO ji = 1, nidx
               zdhice(ji,jl) = h_i_2d(ji,jl) - h_ib_2d(ji,jl)
            END DO
         END DO
         
         ! --- New boundaries for category 1:jpl-1 --- !
         DO jl = 1, jpl - 1
            !
            DO ji = 1, nidx
               !
               ! --- New boundary: Hn* = Hn + Fn*dt --- !
               !     Fn*dt = ( fn + (fn+1 - fn)/(hn+1 - hn) * (Hn - hn) ) * dt = zdhice + zslope * (Hmax - h_i_b)
               !
               IF    ( a_ib_2d(ji,jl) >  epsi10 .AND. a_ib_2d(ji,jl+1) >  epsi10 ) THEN   ! a(jl+1) & a(jl) /= 0
                  zslope           = ( zdhice(ji,jl+1) - zdhice(ji,jl) ) / ( h_ib_2d(ji,jl+1) - h_ib_2d(ji,jl) )
                  zhbnew(ji,jl) = hi_max(jl) + zdhice(ji,jl) + zslope * ( hi_max(jl) - h_ib_2d(ji,jl) )
               ELSEIF( a_ib_2d(ji,jl) >  epsi10 .AND. a_ib_2d(ji,jl+1) <= epsi10 ) THEN   ! a(jl+1)=0 => Hn* = Hn + fn*dt
                  zhbnew(ji,jl) = hi_max(jl) + zdhice(ji,jl)
               ELSEIF( a_ib_2d(ji,jl) <= epsi10 .AND. a_ib_2d(ji,jl+1) >  epsi10 ) THEN   ! a(jl)=0 => Hn* = Hn + fn+1*dt
                  zhbnew(ji,jl) = hi_max(jl) + zdhice(ji,jl+1)
               ELSE                                                                       ! a(jl+1) & a(jl) = 0 
                  zhbnew(ji,jl) = hi_max(jl)
               ENDIF
               !
               ! --- 2 conditions for remapping --- !
               ! 1) hn(t+1)+espi < Hn* < hn+1(t+1)-epsi               
               !    Note: hn(t+1) must not be too close to either HR or HL otherwise a division by nearly 0 is possible 
               !          in ice_itd_glinear in the case (HR-HL) = 3(Hice - HL) or = 3(HR - Hice)
               IF( a_i_2d(ji,jl  ) > epsi10 .AND. h_i_2d(ji,jl  ) > ( zhbnew(ji,jl) - epsi10 ) )   idxice(ji) = 0
               IF( a_i_2d(ji,jl+1) > epsi10 .AND. h_i_2d(ji,jl+1) < ( zhbnew(ji,jl) + epsi10 ) )   idxice(ji) = 0
               
               ! 2) Hn-1 < Hn* < Hn+1  
               IF( zhbnew(ji,jl) < hi_max(jl-1) )   idxice(ji) = 0
               IF( zhbnew(ji,jl) > hi_max(jl+1) )   idxice(ji) = 0
               
            END DO
         END DO
         !
         ! --- New boundaries for category jpl --- !
         DO ji = 1, nidx
            IF( a_i_2d(ji,jpl) > epsi10 ) THEN
               zhbnew(ji,jpl) = MAX( hi_max(jpl-1), 3._wp * h_i_2d(ji,jpl) - 2._wp * zhbnew(ji,jpl-1) )
            ELSE
               zhbnew(ji,jpl) = hi_max(jpl)  
            ENDIF
            
            ! --- 1 additional condition for remapping (1st category) --- !
            ! H0+epsi < h1(t) < H1-epsi 
            !    h1(t) must not be too close to either HR or HL otherwise a division by nearly 0 is possible 
            !    in ice_itd_glinear in the case (HR-HL) = 3(Hice - HL) or = 3(HR - Hice)
            IF( h_ib_2d(ji,1) < ( hi_max(0) + epsi10 ) )   idxice(ji) = 0
            IF( h_ib_2d(ji,1) > ( hi_max(1) - epsi10 ) )   idxice(ji) = 0
         END DO
         !
         !-----------------------------------------------------------------------------------------------
         !  3) Identify cells where remapping
         !-----------------------------------------------------------------------------------------------
         nidx2 = 0 ; idxice2(:) = 0
         DO ji = 1, nidx
            IF( idxice(ji) /= 0 ) THEN
               nidx2 = nidx2 + 1
               idxice2(nidx2) = idxice(ji)
               zhbnew(nidx2,:) = zhbnew(ji,:)  ! adjust zhbnew to new indices
            ENDIF
         END DO
         idxice(:) = idxice2(:)
         nidx      = nidx2
         !
      ENDIF
   
      !-----------------------------------------------------------------------------------------------
      !  4) Compute g(h) 
      !-----------------------------------------------------------------------------------------------
      IF( nidx > 0 ) THEN
         !
         zhb0(:) = hi_max(0)   ;   zhb1(:) = hi_max(1)
         g0(:,:) = 0._wp       ;   g1(:,:) = 0._wp 
         hL(:,:) = 0._wp       ;   hR(:,:) = 0._wp 
         !
         DO jl = 1, jpl
            !
            CALL tab_2d_1d( nidx, idxice(1:nidx), h_ib_1d(1:nidx), h_i_b(:,:,jl) )
            CALL tab_2d_1d( nidx, idxice(1:nidx), h_i_1d (1:nidx), h_i(:,:,jl)   )
            CALL tab_2d_1d( nidx, idxice(1:nidx), a_i_1d  (1:nidx), a_i(:,:,jl)    )
            CALL tab_2d_1d( nidx, idxice(1:nidx), v_i_1d  (1:nidx), v_i(:,:,jl)    )
            !
            IF( jl == 1 ) THEN
               !  
               ! --- g(h) for category 1 --- !
               CALL ice_itd_glinear( zhb0(1:nidx)  , zhb1(1:nidx)  , h_ib_1d(1:nidx)  , a_i_1d(1:nidx)  ,  &   ! in
                  &                  g0  (1:nidx,1), g1  (1:nidx,1), hL      (1:nidx,1), hR    (1:nidx,1)   )   ! out
                  !
               ! Area lost due to melting of thin ice
               DO ji = 1, nidx
                  !
                  IF( a_i_1d(ji) > epsi10 ) THEN
                     !
                     zdh0 =  h_i_1d(ji) - h_ib_1d(ji)                
                     IF( zdh0 < 0.0 ) THEN      !remove area from category 1
                        zdh0 = MIN( -zdh0, hi_max(1) )
                        !Integrate g(1) from 0 to dh0 to estimate area melted
                        zetamax = MIN( zdh0, hR(ji,1) ) - hL(ji,1)
                        !
                        IF( zetamax > 0.0 ) THEN
                           zx1    = zetamax
                           zx2    = 0.5 * zetamax * zetamax 
                           zda0   = g1(ji,1) * zx2 + g0(ji,1) * zx1                        ! ice area removed
                           zdamax = a_i_1d(ji) * (1.0 - h_i_1d(ji) / h_ib_1d(ji) ) ! Constrain new thickness <= h_i                
                           zda0   = MIN( zda0, zdamax )                                                  ! ice area lost due to melting 
                           !     of thin ice (zdamax > 0)
                           ! Remove area, conserving volume
                           h_i_1d(ji) = h_i_1d(ji) * a_i_1d(ji) / ( a_i_1d(ji) - zda0 )
                           a_i_1d(ji)  = a_i_1d(ji) - zda0
                           v_i_1d(ji)  = a_i_1d(ji) * h_i_1d(ji) ! clem-useless ?
                        ENDIF
                        !
                     ELSE ! if ice accretion zdh0 > 0
                        ! zhbnew was 0, and is shifted to the right to account for thin ice growth in openwater (F0 = f1)
                        zhbnew(ji,0) = MIN( zdh0, hi_max(1) ) 
                     ENDIF
                     !
                  ENDIF
                  !
               END DO
               !
               CALL tab_1d_2d( nidx, idxice(1:nidx), h_i_1d (1:nidx), h_i(:,:,jl)   )
               CALL tab_1d_2d( nidx, idxice(1:nidx), a_i_1d  (1:nidx), a_i(:,:,jl)    )
               CALL tab_1d_2d( nidx, idxice(1:nidx), v_i_1d  (1:nidx), v_i(:,:,jl)    )
               !
            ENDIF ! jl=1
            !
            ! --- g(h) for each thickness category --- !  
            CALL ice_itd_glinear( zhbnew(1:nidx,jl-1), zhbnew(1:nidx,jl), h_i_1d(1:nidx)   , a_i_1d(1:nidx)   ,  &   ! in
               &                  g0    (1:nidx,jl  ), g1    (1:nidx,jl), hL     (1:nidx,jl), hR    (1:nidx,jl)   )   ! out
            !
         END DO
         
         !-----------------------------------------------------------------------------------------------
         !  5) Compute area and volume to be shifted across each boundary (Eq. 18)
         !-----------------------------------------------------------------------------------------------
         DO jl = 1, jpl - 1
            !
            DO ji = 1, nidx
               !
               ! left and right integration limits in eta space
               IF (zhbnew(ji,jl) > hi_max(jl)) THEN ! Hn* > Hn => transfer from jl to jl+1
                  zetamin = MAX( hi_max(jl)   , hL(ji,jl) ) - hL(ji,jl)   ! hi_max(jl) - hL 
                  zetamax = MIN( zhbnew(ji,jl), hR(ji,jl) ) - hL(ji,jl)   ! hR - hL
                  jdonor(ji,jl) = jl
               ELSE                                 ! Hn* <= Hn => transfer from jl+1 to jl
                  zetamin = 0.0
                  zetamax = MIN( hi_max(jl), hR(ji,jl+1) ) - hL(ji,jl+1)  ! hi_max(jl) - hL
                  jdonor(ji,jl) = jl + 1
               ENDIF
               zetamax = MAX( zetamax, zetamin ) ! no transfer if etamax < etamin
               !
               zx1  = zetamax - zetamin
               zwk1 = zetamin * zetamin
               zwk2 = zetamax * zetamax
               zx2  = 0.5 * ( zwk2 - zwk1 )
               zwk1 = zwk1 * zetamin
               zwk2 = zwk2 * zetamax
               zx3  = 1.0 / 3.0 * ( zwk2 - zwk1 )
               jcat = jdonor(ji,jl)
               zdaice(ji,jl) = g1(ji,jcat)*zx2 + g0(ji,jcat)*zx1
               zdvice(ji,jl) = g1(ji,jcat)*zx3 + g0(ji,jcat)*zx2 + zdaice(ji,jl)*hL(ji,jcat)
               !
            END DO
         END DO
         
         !----------------------------------------------------------------------------------------------
         ! 6) Shift ice between categories
         !----------------------------------------------------------------------------------------------
         CALL ice_itd_shiftice ( jdonor(1:nidx,:), zdaice(1:nidx,:), zdvice(1:nidx,:) )
         
         !----------------------------------------------------------------------------------------------
         ! 7) Make sure h_i >= minimum ice thickness hi_min
         !----------------------------------------------------------------------------------------------
         CALL tab_2d_1d( nidx, idxice(1:nidx), h_i_1d (1:nidx), h_i(:,:,1)   )
         CALL tab_2d_1d( nidx, idxice(1:nidx), a_i_1d  (1:nidx), a_i(:,:,1)    )
         CALL tab_2d_1d( nidx, idxice(1:nidx), a_ip_1d (1:nidx), a_ip(:,:,1)   )
         
         DO ji = 1, nidx
            IF ( a_i_1d(ji) > epsi10 .AND. h_i_1d(ji) < rn_himin ) THEN
               a_i_1d (ji) = a_i_1d(ji) * h_i_1d(ji) / rn_himin 
               ! MV MP 2016
               IF ( nn_pnd_scheme > 0 ) THEN
                  a_ip_1d(ji) = a_ip_1d(ji) * h_i_1d(ji) / rn_himin
               ENDIF
               ! END MV MP 2016
               h_i_1d(ji) = rn_himin
            ENDIF
         END DO
         !
         CALL tab_1d_2d( nidx, idxice(1:nidx), h_i_1d (1:nidx), h_i(:,:,1)   )
         CALL tab_1d_2d( nidx, idxice(1:nidx), a_i_1d  (1:nidx), a_i(:,:,1)    )
         CALL tab_1d_2d( nidx, idxice(1:nidx), a_ip_1d (1:nidx), a_ip(:,:,1)   )
         !
      ENDIF
      !
      IF( ln_icediachk )   CALL ice_cons_hsm(1, 'iceitd_rem', rdiag_v, rdiag_s, rdiag_t, rdiag_fv, rdiag_fs, rdiag_ft)
      !
   END SUBROUTINE ice_itd_rem


   SUBROUTINE ice_itd_glinear( HbL, Hbr, phice, paice, pg0, pg1, phL, phR )
      !!------------------------------------------------------------------
      !!                ***  ROUTINE ice_itd_glinear ***
      !!
      !! ** Purpose :   build g(h) satisfying area and volume constraints (Eq. 6 and 9)
      !!
      !! ** Method  :   g(h) is linear and written as: g(eta) = g1(eta) + g0
      !!                with eta = h - HL
      !!------------------------------------------------------------------
      REAL(wp), DIMENSION(:), INTENT(in   ) ::   HbL, HbR      ! left and right category boundaries
      REAL(wp), DIMENSION(:), INTENT(in   ) ::   phice, paice  ! ice thickness and concentration
      REAL(wp), DIMENSION(:), INTENT(inout) ::   pg0, pg1      ! coefficients in linear equation for g(eta)
      REAL(wp), DIMENSION(:), INTENT(inout) ::   phL, phR      ! min and max value of range over which g(h) > 0
      !
      INTEGER  ::   ji           ! horizontal indices
      REAL(wp) ::   z1_3 , z2_3  ! 1/3 , 2/3
      REAL(wp) ::   zh13         ! HbL + 1/3 * (HbR - HbL)
      REAL(wp) ::   zh23         ! HbL + 2/3 * (HbR - HbL)
      REAL(wp) ::   zdhr         ! 1 / (hR - hL)
      REAL(wp) ::   zwk1, zwk2   ! temporary variables
      !!------------------------------------------------------------------
      !
      z1_3 = 1._wp / 3._wp
      z2_3 = 2._wp / 3._wp
      !
      DO ji = 1, nidx
         !
         IF( paice(ji) > epsi10  .AND. phice(ji) > 0._wp )  THEN
            !
            ! Initialize hL and hR
            phL(ji) = HbL(ji)
            phR(ji) = HbR(ji)
            !
            ! Change hL or hR if hice falls outside central third of range,
            ! so that hice is in the central third of the range [HL HR]
            zh13 = z1_3 * ( 2._wp * phL(ji) +         phR(ji) )
            zh23 = z1_3 * (         phL(ji) + 2._wp * phR(ji) )
            !
            IF    ( phice(ji) < zh13 ) THEN   ;   phR(ji) = 3._wp * phice(ji) - 2._wp * phL(ji) ! move HR to the left
            ELSEIF( phice(ji) > zh23 ) THEN   ;   phL(ji) = 3._wp * phice(ji) - 2._wp * phR(ji) ! move HL to the right
            ENDIF
            !
            ! Compute coefficients of g(eta) = g0 + g1*eta
            zdhr = 1._wp / (phR(ji) - phL(ji))
            zwk1 = 6._wp * paice(ji) * zdhr
            zwk2 = ( phice(ji) - phL(ji) ) * zdhr
            pg0(ji) = zwk1 * ( z2_3 - zwk2 )                    ! Eq. 14
            pg1(ji) = 2._wp * zdhr * zwk1 * ( zwk2 - 0.5_wp )   ! Eq. 14
            !
         ELSE  ! remap_flag = .false. or a_i < epsi10 
            phL(ji) = 0._wp
            phR(ji) = 0._wp
            pg0(ji) = 0._wp
            pg1(ji) = 0._wp
         ENDIF
         !
      END DO
      !
   END SUBROUTINE ice_itd_glinear


   SUBROUTINE ice_itd_shiftice( kdonor, pdaice, pdvice )
      !!------------------------------------------------------------------
      !!                ***  ROUTINE ice_itd_shiftice ***
      !!
      !! ** Purpose :   shift ice across category boundaries, conserving everything
      !!              ( area, volume, energy, age*vol, and mass of salt )
      !!------------------------------------------------------------------
      INTEGER , DIMENSION(:,:), INTENT(in) ::   kdonor   ! donor category index
      REAL(wp), DIMENSION(:,:), INTENT(in) ::   pdaice   ! ice area transferred across boundary
      REAL(wp), DIMENSION(:,:), INTENT(in) ::   pdvice   ! ice volume transferred across boundary
      !
      INTEGER  ::   ji, jj, jl, jk     ! dummy loop indices
      INTEGER  ::   ii, ij, jl2, jl1   ! local integers
      REAL(wp) ::   ztrans             ! ice/snow transferred
      REAL(wp), DIMENSION(jpij)     ::   zworka, zworkv   ! workspace
      REAL(wp), DIMENSION(jpij,jpl) ::   zaTsfn           !  -    -
      !!------------------------------------------------------------------
         
      CALL tab_3d_2d( nidx, idxice(1:nidx), h_i_2d (1:nidx,1:jpl), h_i   )
      CALL tab_3d_2d( nidx, idxice(1:nidx), a_i_2d  (1:nidx,1:jpl), a_i    )
      CALL tab_3d_2d( nidx, idxice(1:nidx), v_i_2d  (1:nidx,1:jpl), v_i    )
      CALL tab_3d_2d( nidx, idxice(1:nidx), v_s_2d  (1:nidx,1:jpl), v_s    )
      CALL tab_3d_2d( nidx, idxice(1:nidx), oa_i_2d (1:nidx,1:jpl), oa_i   )
      CALL tab_3d_2d( nidx, idxice(1:nidx), smv_i_2d(1:nidx,1:jpl), smv_i  )
      CALL tab_3d_2d( nidx, idxice(1:nidx), a_ip_2d (1:nidx,1:jpl), a_ip   )
      CALL tab_3d_2d( nidx, idxice(1:nidx), v_ip_2d (1:nidx,1:jpl), v_ip   )
      CALL tab_3d_2d( nidx, idxice(1:nidx), t_su_2d (1:nidx,1:jpl), t_su   )

      !----------------------------------------------------------------------------------------------
      ! 1) Define a variable equal to a_i*T_su
      !----------------------------------------------------------------------------------------------
      DO jl = 1, jpl
         DO ji = 1, nidx
            zaTsfn(ji,jl) = a_i_2d(ji,jl) * t_su_2d(ji,jl)
         END DO
      END DO
      
      !-------------------------------------------------------------------------------
      ! 2) Transfer volume and energy between categories
      !-------------------------------------------------------------------------------
      DO jl = 1, jpl - 1
         DO ji = 1, nidx
            !
            jl1 = kdonor(ji,jl)
            !
            IF( jl1 > 0 ) THEN
               !
               IF ( jl1 == jl  ) THEN   ;   jl2 = jl1+1
               ELSE                     ;   jl2 = jl 
               ENDIF
               !
               IF( v_i_2d(ji,jl1) >= epsi10 ) THEN   ;   zworkv(ji) = pdvice(ji,jl) / v_i_2d(ji,jl1)
               ELSE                                  ;   zworkv(ji) = 0._wp
               ENDIF
               IF( a_i_2d(ji,jl1) >= epsi10 ) THEN   ;   zworka(ji) = pdaice(ji,jl) / a_i_2d(ji,jl1)
               ELSE                                  ;   zworka(ji) = 0._wp
               ENDIF
               !
               a_i_2d(ji,jl1) = a_i_2d(ji,jl1) - pdaice(ji,jl)       ! Ice areas
               a_i_2d(ji,jl2) = a_i_2d(ji,jl2) + pdaice(ji,jl)
               !
               v_i_2d(ji,jl1) = v_i_2d(ji,jl1) - pdvice(ji,jl)       ! Ice volumes
               v_i_2d(ji,jl2) = v_i_2d(ji,jl2) + pdvice(ji,jl)
               !
               ztrans         = v_s_2d(ji,jl1) * zworkv(ji)          ! Snow volumes
               v_s_2d(ji,jl1) = v_s_2d(ji,jl1) - ztrans
               v_s_2d(ji,jl2) = v_s_2d(ji,jl2) + ztrans 
               !          
               !                                                     ! Ice age 
               ztrans             = oa_i_2d(ji,jl1) * pdaice(ji,jl)  !!clem: should be * zworka(ji) but it does not work ????
               oa_i_2d(ji,jl1)    = oa_i_2d(ji,jl1) - ztrans
               oa_i_2d(ji,jl2)    = oa_i_2d(ji,jl2) + ztrans
               !
               ztrans             = smv_i_2d(ji,jl1) * zworkv(ji)    ! Ice salinity
               !
               smv_i_2d(ji,jl1)   = smv_i_2d(ji,jl1) - ztrans
               smv_i_2d(ji,jl2)   = smv_i_2d(ji,jl2) + ztrans
               !
               !                                                     ! Surface temperature
               ztrans          = t_su_2d(ji,jl1) * pdaice(ji,jl)     !!clem: should be * zworka(ji) but it does not work ????
               zaTsfn(ji,jl1)  = zaTsfn(ji,jl1) - ztrans
               zaTsfn(ji,jl2)  = zaTsfn(ji,jl2) + ztrans
               !  
               ! MV MP 2016 
               IF ( nn_pnd_scheme > 0 ) THEN
                  !                                                  ! Pond fraction
                  ztrans          = a_ip_2d(ji,jl1) * pdaice(ji,jl) !!clem: should be * zworka(ji) but it does not work
                  a_ip_2d(ji,jl1) = a_ip_2d(ji,jl1) - ztrans
                  a_ip_2d(ji,jl2) = a_ip_2d(ji,jl2) + ztrans
                  !                                                  ! Pond volume (also proportional to da/a)
                  ztrans          = v_ip_2d(ji,jl1) * pdaice(ji,jl) !!clem: should be * zworka(ji) but it does not work
                  v_ip_2d(ji,jl1) = v_ip_2d(ji,jl1) - ztrans
                  v_ip_2d(ji,jl2) = v_ip_2d(ji,jl2) + ztrans
               ENDIF
               ! END MV MP 2016
               !
            ENDIF   ! jl1 >0
         END DO
         !
         DO jk = 1, nlay_s         !--- Snow heat content
            !
            DO ji = 1, nidx
               ii = MOD( idxice(ji) - 1, jpi ) + 1
               ij = ( idxice(ji) - 1 ) / jpi + 1
               !
               jl1 = kdonor(ji,jl)
               !
               IF( jl1 > 0 ) THEN
                  IF(jl1 == jl) THEN  ;  jl2 = jl+1
                  ELSE                ;  jl2 = jl
                  ENDIF
                  !
                  ztrans            = e_s(ii,ij,jk,jl1) * zworkv(ji)
                  e_s(ii,ij,jk,jl1) = e_s(ii,ij,jk,jl1) - ztrans
                  e_s(ii,ij,jk,jl2) = e_s(ii,ij,jk,jl2) + ztrans
               ENDIF
            END DO
         END DO

         DO jk = 1, nlay_i         !--- Ice heat content
            DO ji = 1, nidx
               ii = MOD( idxice(ji) - 1, jpi ) + 1
               ij = ( idxice(ji) - 1 ) / jpi + 1
               !
               jl1 = kdonor(ji,jl)
               !
               IF( jl1 > 0 ) THEN
                  IF(jl1 == jl) THEN  ;  jl2 = jl+1
                  ELSE                ;  jl2 = jl
                  ENDIF
                  !
                  ztrans            = e_i(ii,ij,jk,jl1) * zworkv(ji)
                  e_i(ii,ij,jk,jl1) = e_i(ii,ij,jk,jl1) - ztrans
                  e_i(ii,ij,jk,jl2) = e_i(ii,ij,jk,jl2) + ztrans
               ENDIF
            END DO
         END DO
         !
      END DO                   ! boundaries, 1 to jpl-1
      
      !-------------------------------------------------------------------------------
      ! 3) Update ice thickness and temperature
      !-------------------------------------------------------------------------------
      WHERE( a_i_2d(1:nidx,:) >= epsi20 )
         h_i_2d(1:nidx,:)  =  v_i_2d(1:nidx,:) / a_i_2d(1:nidx,:) 
         t_su_2d(1:nidx,:)  =  zaTsfn(1:nidx,:) / a_i_2d(1:nidx,:) 
      ELSEWHERE
         h_i_2d(1:nidx,:)  = 0._wp
         t_su_2d(1:nidx,:)  = rt0
      END WHERE
      !
      CALL tab_2d_3d( nidx, idxice(1:nidx), h_i_2d (1:nidx,1:jpl), h_i  )
      CALL tab_2d_3d( nidx, idxice(1:nidx), a_i_2d  (1:nidx,1:jpl), a_i   )
      CALL tab_2d_3d( nidx, idxice(1:nidx), v_i_2d  (1:nidx,1:jpl), v_i   )
      CALL tab_2d_3d( nidx, idxice(1:nidx), v_s_2d  (1:nidx,1:jpl), v_s   )
      CALL tab_2d_3d( nidx, idxice(1:nidx), oa_i_2d (1:nidx,1:jpl), oa_i  )
      CALL tab_2d_3d( nidx, idxice(1:nidx), smv_i_2d(1:nidx,1:jpl), smv_i )
      CALL tab_2d_3d( nidx, idxice(1:nidx), a_ip_2d (1:nidx,1:jpl), a_ip  )
      CALL tab_2d_3d( nidx, idxice(1:nidx), v_ip_2d (1:nidx,1:jpl), v_ip  )
      CALL tab_2d_3d( nidx, idxice(1:nidx), t_su_2d (1:nidx,1:jpl), t_su  )
      !
   END SUBROUTINE ice_itd_shiftice
   

   SUBROUTINE ice_itd_reb( kt )
      !!------------------------------------------------------------------
      !!                ***  ROUTINE ice_itd_reb ***
      !!
      !! ** Purpose : rebin - rebins thicknesses into defined categories
      !!
      !! ** Method  : If a category thickness is out of bounds, shift part (for down to top)
      !!              or entire (for top to down) area, volume, and energy
      !!              to the neighboring category
      !!------------------------------------------------------------------
      INTEGER , INTENT (in) ::   kt      ! Ocean time step 
      INTEGER ::   ji, jj, jl   ! dummy loop indices
      !
      INTEGER , DIMENSION(jpij,jpl-1) ::   jdonor           ! donor category index
      REAL(wp), DIMENSION(jpij,jpl-1) ::   zdaice, zdvice   ! ice area and volume transferred
      !!------------------------------------------------------------------
      !
      IF( kt == nit000 .AND. lwp )   WRITE(numout,*) '-- ice_itd_reb: rebining ice thickness distribution' 

      jdonor(:,:) = 0
      zdaice(:,:) = 0._wp
      zdvice(:,:) = 0._wp
      !
      !                       !---------------------------------------
      DO jl = 1, jpl-1        ! identify thicknesses that are too big
         !                    !---------------------------------------
         nidx = 0   ;   idxice(:) = 0
         DO jj = 1, jpj
            DO ji = 1, jpi
               IF( a_i(ji,jj,jl) > epsi10 .AND. v_i(ji,jj,jl) > (a_i(ji,jj,jl) * hi_max(jl)) ) THEN
                  nidx = nidx + 1
                  idxice( nidx ) = (jj - 1) * jpi + ji                  
               ENDIF
            END DO
         END DO
         !
!!clem   CALL tab_2d_1d( nidx, idxice(1:nidx), h_i_1d (1:nidx), h_i(:,:,jl)   )
         CALL tab_2d_1d( nidx, idxice(1:nidx), a_i_1d  (1:nidx), a_i(:,:,jl)    )
         CALL tab_2d_1d( nidx, idxice(1:nidx), v_i_1d  (1:nidx), v_i(:,:,jl)    )
         !
         DO ji = 1, nidx
            jdonor(ji,jl)  = jl 
            ! how much of a_i you send in cat sup is somewhat arbitrary
!!clem: these do not work properly after a restart (I do not know why)
!!          zdaice(ji,jl)  = a_i_1d(ji) * ( h_i_1d(ji) - hi_max(jl) + epsi10 ) / h_i_1d(ji)  
!!          zdvice(ji,jl)  = v_i_1d(ji) - ( a_i_1d(ji) - zdaice(ji,jl) ) * ( hi_max(jl) - epsi10 )
!!clem: these do not work properly after a restart (I do not know why)
!!            zdaice(ji,jl)  = a_i_1d(ji)
!!            zdvice(ji,jl)  = v_i_1d(ji)
!!clem: these are from UCL and work ok
            zdaice(ji,jl)  = a_i_1d(ji) * 0.5_wp
            zdvice(ji,jl)  = v_i_1d(ji) - zdaice(ji,jl) * ( hi_max(jl) + hi_max(jl-1) ) * 0.5_wp
         END DO
         !
         IF( nidx > 0 ) THEN
            CALL ice_itd_shiftice( jdonor(1:nidx,:), zdaice(1:nidx,:), zdvice(1:nidx,:) )  ! Shift jl=>jl+1
            ! Reset shift parameters
            jdonor(1:nidx,jl) = 0
            zdaice(1:nidx,jl) = 0._wp
            zdvice(1:nidx,jl) = 0._wp
         ENDIF
         !
      END DO

      !                       !-----------------------------------------
      DO jl = jpl-1, 1, -1    ! Identify thicknesses that are too small
         !                    !-----------------------------------------
         nidx = 0 ; idxice(:) = 0
         DO jj = 1, jpj
            DO ji = 1, jpi
               IF( a_i(ji,jj,jl+1) > epsi10 .AND. v_i(ji,jj,jl+1) <= (a_i(ji,jj,jl+1) * hi_max(jl)) ) THEN
                  nidx = nidx + 1
                  idxice( nidx ) = (jj - 1) * jpi + ji                  
               ENDIF
            END DO
         END DO
         !
         CALL tab_2d_1d( nidx, idxice(1:nidx), a_i_1d  (1:nidx), a_i(:,:,jl+1)    ) ! jl+1 is ok
         CALL tab_2d_1d( nidx, idxice(1:nidx), v_i_1d  (1:nidx), v_i(:,:,jl+1)    ) ! jl+1 is ok
         DO ji = 1, nidx
            jdonor(ji,jl) = jl + 1
            zdaice(ji,jl) = a_i_1d(ji) 
            zdvice(ji,jl) = v_i_1d(ji)
         END DO
         !
         IF( nidx > 0 ) THEN
            CALL ice_itd_shiftice( jdonor(1:nidx,:), zdaice(1:nidx,:), zdvice(1:nidx,:) )  ! Shift jl+1=>jl
            ! Reset shift parameters
            jdonor(1:nidx,jl) = 0
            zdaice(1:nidx,jl) = 0._wp
            zdvice(1:nidx,jl) = 0._wp
         ENDIF
         !
      END DO
      !
   END SUBROUTINE ice_itd_reb

   SUBROUTINE ice_itd_init
      !!------------------------------------------------------------------
      !!                ***  ROUTINE ice_itd_init ***
      !!
      !! ** Purpose :   Initializes the ice thickness distribution
      !! ** Method  :   ...
      !! ** input   :   Namelist namitd
      !!-------------------------------------------------------------------
      INTEGER  ::   jl    ! dummy loop index
      INTEGER  ::   ios   ! Local integer output status for namelist read
      REAL(wp) ::   zhmax, znum, zden, zalpha   !   -      -
      !!
      NAMELIST/namitd/ rn_himean, rn_himin
      !!------------------------------------------------------------------
      !
      REWIND( numnam_ice_ref )      ! Namelist namitd in reference namelist : Parameters for ice
      READ  ( numnam_ice_ref, namitd, IOSTAT = ios, ERR = 901)
901   IF( ios /= 0 ) CALL ctl_nam ( ios , 'namitd in reference namelist', lwp )

      REWIND( numnam_ice_cfg )      ! Namelist namitd in configuration namelist : Parameters for ice
      READ  ( numnam_ice_cfg, namitd, IOSTAT = ios, ERR = 902 )
902   IF( ios /= 0 ) CALL ctl_nam ( ios , 'namitd in configuration namelist', lwp )
      IF(lwm) WRITE ( numoni, namitd )
      !
      IF(lwp) THEN                  ! control print
         WRITE(numout,*)
         WRITE(numout,*) 'ice_itd_init: Initialization of ice cat distribution '
         WRITE(numout,*) '~~~~~~~~~~~~'
         WRITE(numout,*) '   Namelist namitd: '
         WRITE(numout,*) '      mean ice thickness in the domain               rn_himean = ', rn_himean
         WRITE(numout,*) '      minimum ice thickness                          rn_himin  = ', rn_himin 
      ENDIF
      !
      !-----------------------------------!
      !  Thickness categories boundaries  !
      !-----------------------------------!
      !
      zalpha = 0.05_wp              ! max of each category (from h^(-alpha) function)
      zhmax  = 3._wp * rn_himean
      DO jl = 1, jpl
         znum = jpl * ( zhmax+1 )**zalpha
         zden = REAL( jpl-jl , wp ) * ( zhmax + 1._wp )**zalpha + REAL( jl , wp )
         hi_max(jl) = ( znum / zden )**(1./zalpha) - 1
      END DO
      !
      DO jl = 1, jpl                ! mean thickness by category
         hi_mean(jl) = ( hi_max(jl) + hi_max(jl-1) ) * 0.5_wp
      END DO
      !
      hi_max(jpl) = 99._wp          ! set to a big value to ensure that all ice is thinner than hi_max(jpl)
      !
      IF(lwp) WRITE(numout,*)
      IF(lwp) WRITE(numout,*) '   ===>>>   resulting thickness category boundaries :'
      IF(lwp) WRITE(numout,*) '            hi_max(:)= ', hi_max(0:jpl)
      !
   END SUBROUTINE ice_itd_init

#else
   !!----------------------------------------------------------------------
   !!   Default option :         Empty module         NO ESIM sea-ice model
   !!----------------------------------------------------------------------
#endif

   !!======================================================================
END MODULE iceitd
